from django.db import models
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
#from blogger.settings import BLOG_SETTINGS
from django.conf import settings
from django.db.models import Sum
from blogger.managers import PostManager
from django.contrib.syndication.views import Feed
import unidecode
import re
from taggit.managers import TaggableManager

from django.utils.translation import ugettext_lazy as _
BLOG_SETTINGS = settings.BLOG_SETTINGS['defaults']

class Author(models.Model):
    """
    Association of a Blog Author data and a django user
    """

    first_name = models.CharField(max_length=200, verbose_name=_("first name"))
    last_name = models.CharField(max_length=200, verbose_name=_("last name"))
    user = models.ForeignKey(User, verbose_name=_("user"))

    class Meta:
        unique_together = (("first_name", "last_name"),)

    def __unicode__(self):
        return ' '.join([self.first_name, self.last_name])

    def get_absolute_url(self):
        return reverse('author_archive',
                       args=['-'.join([self.first_name, self.last_name])])

    def number_of_posts(self):
        """Return count of post_set"""
        return self.post_set.count()
    number_of_posts.short_description = _("Number of posts")


class Post(models.Model):
    """
    User generated blog post
    """

    author = models.ForeignKey(Author, blank=True, verbose_name=_("author"))
    title = models.CharField(max_length=200, unique=True,
                             verbose_name=_("title"))
    body = models.TextField(verbose_name=_("body"))
    created_at = models.DateTimeField(auto_now_add=True,
                                      verbose_name=_("created at"))
    published = models.BooleanField(default=BLOG_SETTINGS['auto_publish'],
                                    verbose_name=_("published?"))
    promoted = models.BooleanField(default=BLOG_SETTINGS['auto_promote'],
                                    verbose_name=_("promoted?"))
    tags = TaggableManager() #models.ManyToManyField(Tag, blank=True, verbose_name=_("tags"))
    slug = models.SlugField(max_length=200, unique=True,
                            verbose_name=_("slug"))

    objects = models.Manager()  # The default manager.
    popular_posts = PostManager()

    class Meta:
        verbose_name = _("post")
        verbose_name_plural = _("posts")

    def __unicode__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('view_post', args=[str(self.slug)])

    def get_tags(self):
        """Returns tags as a composite string"""
        names = ', '.join([t.name for t in self.tags.all()])
        #if len(names) > 20:
        #    names = names[:20] + "..."
        return names
    get_tags.short_description = _("Tags")

    # slug field should remove the need for this.
    # Leaving it here for a while until I'm sure
    # def set_slug(self):
    #     """Sets self.slug from self.title"""
    #     title_str = unidecode.unidecode(self.title).lower()
    #     self.slug = re.sub(r'\W+','-',title_str)



from django.db import models
from django.contrib.contenttypes.models import ContentType


#http://stackoverflow.com/questions/8215570/ordering-entries-via-comment-count-with-django
#https://docs.djangoproject.com/en/dev/topics/db/managers/#custom-managers-and-model-inheritance
class PostManager(models.Manager):
    """A custom manager for Post models"""
    entry_type = None

    def get_query_set(self):
        """attaches django comment_count to all querysets"""
        if not self.entry_type:
            self.entry_type = ContentType.objects.get(model='post')

        return super(PostManager, self).get_query_set().all().extra(select={
            'comment_count': """SELECT COUNT(*) FROM django_comments
            WHERE django_comments.object_pk = blogger_post.id
            AND django_comments.content_type_id = %s"""
            }, select_params=(self.entry_type.id,)).order_by('-comment_count')



from django.contrib.syndication.views import Feed
from django.core.urlresolvers import reverse
from blogger.models import Post
from taggit.models import Tag
from django.utils.translation import ugettext_lazy as _


class LatestEntriesFeed(Feed):
    """RSS feed of the latest Posts"""
    title = _("Blog RSS")
    link = "/rss/"
    description = _("Updates on changes and additions to this blog.")

    def items(self):
        return Post.objects.order_by('-created_at')[:5]

    def item_author_name(self, item):
        """
        Takes an item, as returned by items(), and returns the item's
        author's name as a normal Python string.
        """
        return item.author

    def item_title(self, item):
        """Returns an item's title"""
        return item.title

    def item_description(self, item):
        return item.body

    def item_pubdate(self, item):
        """
        Takes an item, as returned by items(), and returns the item's
        pubdate.
        """
        return item.created_at

    def item_categories(self):
        """
        Returns the categories for every item in the feed.
        """
        return Tag.objects.all()

    # item_link is only needed if Post has no get_absolute_url method.
    def item_link(self, item):
        """Returns item URL"""
        return reverse('view_post', args=[item.slug])



from rest_framework import permissions


class IsOwnerOrReadOnly(permissions.BasePermission):
    """
    Custom permission to only allow owners of an object to edit it.
    """

    def has_permission(self, request, view, obj=None):
        # Skip the check unless this is an object-level test
        if obj is None:
            return True

        # Read permissions are allowed to any request
        if request.method in permissions.SAFE_METHODS:
            return True

        # Write permissions are only allowed to the owner of the snippet
        return obj.author.user == request.user



from django.contrib import admin
from blogger.models import Post, Author


class PostAdmin(admin.ModelAdmin):
    """Admin panel class for Post"""
    exclude = ('author',)
    date_hierarchy = 'created_at'
    list_display = ('title', 'author', 'created_at', 'published', 'get_tags')
    list_filter = ['author', 'created_at', 'published']
    search_fields = ('title', 'body')
    prepopulated_fields = {"slug": ("title",)}

    #http://stackoverflow.com/questions/753704/manipulating-data-in-djangos-admin-panel-on-save
    def save_model(self, request, obj, form, change):
        """Customize save method via admin panel save"""
        if not change:
            obj.author = request.user.author_set.all()[0]
        #obj.set_slug()
        obj.save()


class AuthorAdmin(admin.ModelAdmin):
    """Admin panel class for Author"""
    list_display = ('first_name', 'last_name', 'user', 'number_of_posts')
    search_fields = ('first_name', 'last_name')

admin.site.register(Post, PostAdmin)
admin.site.register(Author, AuthorAdmin)



from django.conf.urls import patterns, include, url
from blogger.feeds import LatestEntriesFeed
from rest_framework.urlpatterns import format_suffix_patterns
from blogger.views import PostList, PostDetail, AuthorList, AuthorDetail

api_patterns = patterns('blogger.views',
    url(r'^$', 'api_root'),
    url(r'^posts/$', PostList.as_view(), name='posts-list'),
    url(r'^posts/(?P<pk>\d+)/$', PostDetail.as_view(), name='post-detail'),
    url(r'^authors/$', AuthorList.as_view(), name='authors-list'),
    url(r'^authors/(?P<pk>\d+)/$', AuthorDetail.as_view(), name='author-detail'),
)

# Format suffixes
api_patterns = format_suffix_patterns(api_patterns, allowed=['json', 'api'])

urlpatterns = patterns('blogger.views',
    url(r'^$', 'list', name='all_archive'),
    url(r'^(?P<year>\d{4})/$', 'list', name="yearly_archive"),
    url(r'^(?P<year>\d{4})/(?P<month>\d{2})/$', 'list', name="monthly_archive"),
    url(r'^tag/(?P<tag>[\w-]+)/$', 'list', name='tag_archive'),
    url(r'^author/(?P<author>[\w-]+)/$', 'list', name='author_archive'),
    url(r'^confirm/comment/$', 'comment_posted', name='comment_posted'),
    url(r'^post/(?P<slug>[\w-]+)/$', 'view_post', name='view_post'),
    (r'^rss/$', LatestEntriesFeed()),
    (r'^api/', include(api_patterns)),  
    url(r'^api-auth/', include('rest_framework.urls', namespace='rest_framework'))
)



from django.http import HttpRequest, HttpResponse
from django.shortcuts import render_to_response, redirect
from django.core.urlresolvers import reverse
from django.template import RequestContext
from django.db.models import Sum
from django.contrib import messages
from django.contrib.comments import Comment
from rest_framework import generics
from rest_framework.decorators import api_view
from rest_framework.reverse import reverse
from rest_framework.response import Response
from rest_framework import permissions
from blogger.models import Post, Author
from blogger.permissions import IsOwnerOrReadOnly
from blogger.serializers import PostSerializer, AuthorSerializer
from taggit.models import Tag
from django.utils.translation import ugettext_lazy as _
import datetime


# builds the dictionary for secondary navigation
# by post title
def get_sidebar_data():
    promoted_posts = Post.objects.filter(
        published=True, promoted=True).order_by('-created_at')
    popular_posts = Post.popular_posts.filter(
        published=True).order_by('-created_at')[:5]
    recent_posts = Post.objects.filter(
        published=True).order_by('-created_at')[:5]
    archive = Post.objects.filter(
        published=True).dates('created_at', 'month', order='DESC')
    tags = Tag.objects.all()
    authors = Author.objects.all()
    data = {
        'promoted_posts': promoted_posts,
        'popular_posts': popular_posts,
        'recent_posts': recent_posts,
        'archive': archive,
        'tags': tags,
        'authors': authors,
    }
    return data


# renders data onto list.html for the current theme
def render_on_list(request, data):

    return render_to_response('list.html', data,
                              context_instance=RequestContext(request)
                              )


# redirects to the item a comment was made on
# when a user posts a comment.
def comment_posted(request):

    c = request.GET['c']
    messages.add_message(request, messages.SUCCESS,
                         _('Commented added successfully.'))
    c = Comment.objects.get(pk=c)
    return redirect(c.content_object)


# returns a list of posts onto the list template
def list(request, year=None, month=None, tag=None, author=None):

    sidebar_data = get_sidebar_data()

    data = {
        'posts': None,
        'section_title': _('Posts')
    }

    data.update(sidebar_data)

    if tag:
        posts = Post.objects.filter(published=True, tags__slug=tag)
        data['posts'] = posts
        data['section_title'] = _("Tag archive")
        return render_on_list(request, data)
    if author:
        fname, lname = author.split('-')
        posts = Post.objects.filter(published=True,
                                    author__first_name=fname,
                                    author__last_name=lname
                                    ).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = _("Author archive")
        return render_on_list(request, data)
    if not year:
        posts = Post.objects.filter(published=True).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = _("Posts")
        return render_on_list(request, data)
        #Recent posts reverse chrono
    if not month:
        posts = Post.objects.filter(published=True,
                                    created_at__year=year
                                    ).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = _("Yearly Archive")
        return render_on_list(request, data)
    else:
        posts = Post.objects.filter(published=True,
                                    created_at__year=year,
                                    created_at__month=month
                                    ).order_by('-created_at')
        data['posts'] = posts
        data['section_title'] = _("Monthly Archive")
        return render_on_list(request, data)


# renders a single post
def view_post(request, slug):

    post = Post.objects.get(slug=slug)
    if not post.published:
        messages.add_message(request, messages.SUCCESS,
                             _('Post does not exist or is not published yet.'))
        redirect(reverse('all_archive'))

    sidebar_data = get_sidebar_data()
    data = {
        'post': post,
    }
    data.update(sidebar_data)
    return render_to_response('view_post.html', data,
                              context_instance=RequestContext(request)
                              )


def archive_time(request):
    pass


def archive_category(request, category=None):
    pass


def archive_author(request, author=None):
    pass


# Begin API Views
@api_view
def api_root(request, format=None):
    return Response({
        'authors': reverse('authors-list', request=request),
        'posts': reverse('posts-list', request=request),
    })


class AuthorList(generics.ListAPIView):
    """
    API endpoint that represents a list of users.
    """
    model = Author
    serializer_class = AuthorSerializer


class AuthorDetail(generics.RetrieveAPIView):
    """
    API endpoint that represents a single user.
    """
    model = Author
    serializer_class = AuthorSerializer


class PostList(generics.ListCreateAPIView):
    """
    API endpoint that represents a list of groups.
    """
    model = Post
    serializer_class = PostSerializer
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,
                          IsOwnerOrReadOnly)

    def pre_save(self, obj):
        author = Author.objects.get(user=self.request.user)
        obj.author = author


class PostDetail(generics.RetrieveUpdateDestroyAPIView):
    """
    API endpoint that represents a single group.
    """
    model = Post
    serializer_class = PostSerializer
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,
                          IsOwnerOrReadOnly)

    def pre_save(self, obj):
        author = Author.objects.get(user=self.request.user)
        obj.author = author



from rest_framework import serializers
from blogger.models import Post, Author


class PostSerializer(serializers.HyperlinkedModelSerializer):
    """Serializes the Post model"""
    author = serializers.HyperlinkedRelatedField(source='author', view_name=
                                                 'author-detail')
    tags = serializers.ManyRelatedField(source='tags')

    class Meta:
        model = Post
        fields = ('url', 'author', 'title', 'body',
                  'created_at', 'published', 'tags',
                  'slug')

    def restore_object(self, attrs, instance=None):
        if instance is not None:
            instance.author = attrs['author']
            instance.title = attrs['title']
            instance.body = attrs['body']
            instance.published = attrs['published']
            #instance.tags = attrs['tags']
            instance.slug = attrs['slug']
            return instance
        return Post(**attrs)


class AuthorSerializer(serializers.HyperlinkedModelSerializer):
    """Serializes the Author model"""

    class Meta:
        model = Author
        fields = ('url', 'first_name', 'last_name')
